/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.dupfinder;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;


class JDupFinderProgressTask extends ProgressTask {
	private String []	directories	= null;
	private boolean	hiddenFiles	= false;
	private int 		maximum		= 100;
	private int 		current 		= 0;
	private String		currentText 	= "...";
	private boolean	running		= true;
	
	// for the collected data
	private Vector<TreeSet<JDupFinderFile>>	duplicateFiles = new Vector<TreeSet<JDupFinderFile>>();
	
	
	public JDupFinderProgressTask(String [] directories, boolean includeHiddenFiles)  {
		this.directories = directories;
		this.hiddenFiles = includeHiddenFiles;
	}

	
	public void cancel()  {
		running = false;
	}

	
	public int getCurrent()  {
		return current;
	}

	
	public java.lang.String getCurrentText()  {
		return currentText;
	}

	
	public Vector<TreeSet<JDupFinderFile>> getDuplicateFiles()  {
		return duplicateFiles;
	}

	
	public int getMaximum()  {
		return maximum;
	}

	
	public int getMinimum()  {
		return 0;
	}

	
	public void run()  {
		final TreeSet<JDupFinderFile> 	files 					= new TreeSet<JDupFinderFile>();
		final TreeSet<String>			directoriesToRead	= new TreeSet<String>();
		final TreeSet<String>			directoriesRead		= new TreeSet<String>();
		
		// init
		duplicateFiles.clear();
		
		// copy data into tree set
		for (int i=0; i<directories.length; ++i) directoriesToRead.add(directories[i]);
		
		// read directories
		while (directoriesToRead.size() > 0 && running)
			{
			// get element
			String dirname = (String) directoriesToRead.first();
			directoriesToRead.remove(dirname);
			directoriesRead.add(dirname);
			// set text for gui
			current = 0;
			currentText = Helper.GRS("ProgressTracker.Message.ReadingDirectory", dirname);
			// get files in directory
			final File directory = new File(dirname);
			if (! directory.exists()) { JReqTools.showError((JFrame)null, Helper.GRS("Errors.DirectoryDoesNotExist", directory.toString())); continue; }
			else if (! directory.isDirectory()) { JReqTools.showError((JFrame)null, Helper.GRS("Errors.NotADirectory", directory.toString())); continue; }
			String [] names = directory.list(new FilenameFilter() {
				public boolean accept(File dir, String name) {
					if (hiddenFiles) return true;
					if (name.startsWith(".")) return false;
					return true;
				}
			});
			for (int i=0; i<names.length; ++i)
				{
				try
					{
					final File file = new File(dirname, names[i]);
					final String filename = file.getCanonicalPath();
					if (file.isDirectory())
						{
						if (directoriesRead.contains(filename)) System.out.println("Already processed: " + filename);
						else directoriesToRead.add(filename);
						}
					else files.add(new JDupFinderFile(filename));
					}
				catch (IOException x) { System.err.println("JDupFinderProgressTask.run(): " + x); }
				}
			}
		
		// set the maximum
		maximum = files.size();
		
		// compare the files
		while (files.size() > 0 && running)
			{
			// get the first file
			final JDupFinderFile firstFile = (JDupFinderFile) files.first();
			files.remove(firstFile);
			// set the message texts
			current = maximum - files.size();
			currentText = Helper.GRS("ProgressTracker.Message.Processing", ""+(maximum-files.size()), ""+maximum, firstFile.getFilePart());
			// compare files with the same length
			final Vector<JDupFinderFile> sameFiles = new Vector<JDupFinderFile>();
			for (Iterator iterator = files.iterator(); iterator.hasNext(); )
				{
				// get the next file
				JDupFinderFile nextFile = (JDupFinderFile) iterator.next();
				// if length is different, we can stop to compare
				if (nextFile.length != firstFile.length) break;
				// if they are not equal, just continue with the next file
				if (JDupFinderFile.areDifferent(firstFile, nextFile)) continue;
				// they are equal, so add it to the sameFiles vector
				sameFiles.addElement(nextFile);
				}
			// if there are equal files, store them
			if (sameFiles.size() > 0)
				{
				// remove all equal files from the files list
				for (int i=0; i<sameFiles.size(); ++i) files.remove(sameFiles.elementAt(i));
				// add the first element to the list
				sameFiles.insertElementAt(firstFile,0);
				// add the data to the set containg all equal files
				final TreeSet<JDupFinderFile> set = new TreeSet<JDupFinderFile>();
				set.addAll(sameFiles);
				duplicateFiles.addElement(set);
				}
			}
	}

}
