/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.dupfinder;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;


class JDupFinderResultFrame extends HFrame {
	// App Prefs
	private ApplicationPreferences prefs = null;
	
	// Variables
	private Vector<TreeSet<JDupFinderFile>> duplicates = null;
	
	// Widgets
	private final JList		list1			= new JList();
	private final JList		list2			= new JList();
	private final JButton 	showButton 	= new JButton(Helper.GRS("ResultFrame.Button.Show"), HIconFactory.getButtonIcon("apply"));
	private final JButton 	deleteButton 	= new JButton(Helper.GRS("ResultFrame.Button.Delete"), HIconFactory.getButtonIcon("cancel"));
	
	
	public JDupFinderResultFrame(Vector<TreeSet<JDupFinderFile>> duplicateFiles)  {
		super(Helper.GRS("ResultFrame.Title"));
		
		// init
		this.duplicates = duplicateFiles;
		
		// Properties
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		list1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list2.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		// Listeners
		list1.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) { updateList2(e); }});
		list2.addMouseListener(new MouseAdapter() { public void mouseClicked(MouseEvent e) { if (e.getClickCount() == 2) showFiles(); }});
		showButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { showFiles(); }});
		deleteButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { deleteFiles(); }});
		
		// create and set panel
		final JPanel panel = createContentPanel();
		panel.setBorder(UIPreferences.getFrameBorder());
		setContentPane(panel);
		
		// set data
		final TreeSet<JDupFinderFile> set = new TreeSet<JDupFinderFile>();
		for (TreeSet<JDupFinderFile> dup : duplicates) set.add(dup.first());
		list1.setListData(set.toArray());
		list1.setSelectedIndex(0);
		
		// calculate frame size
		pack();
		JDupFinder.addPrefs("ResultFrame", this);
	}

	
	JPanel createContentPanel()  {
		final JPanel panel = new JPanel(new GridBagLayout());
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// Lists
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(new JScrollPane(list1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		c.gridx = 1; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(new JScrollPane(list2, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		
		// Button
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
		panel.add(new JPanel(), c);
		c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(deleteButton, c);
		c.gridx = 3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(showButton, c);
		
		return panel;
	}

	
	void deleteFiles()  {
		// get selected items
		final Object [] selected = list2.getSelectedValues();
		if (selected == null) return;
		
		// delete each files
		for (int i=0; i<selected.length; ++i)
			{
			final JDupFinderFile file = (JDupFinderFile) selected[i];
			final File f = new File(file.filename);
			if (f.delete())
				{
				// close frame if necessary
				if (file.frame != null && file.frame.isVisible()) file.frame.setVisible(false);
				// update the data model
				final int index = list1.getSelectedIndex(); // is always >= 0 !!
				// get the set
				final TreeSet data = (TreeSet)duplicates.elementAt(index);
				data.remove(file);
				list2.setListData(data.toArray());
				}
			else JReqTools.showError(this, Helper.GRS("Error.CouldNotDelete", file.filename));
			}
		
		// if there is only one item left select next possible duplicate's entry
		if (list2.getModel().getSize() < 2)
			{
			final int selectedIndex = list1.getSelectedIndex();
			if (selectedIndex < list1.getModel().getSize() - 1) 
				{
				final Object toSelect = list1.getModel().getElementAt(selectedIndex+1);
				list1.setSelectedValue(toSelect, true);
				}
			}
	}

	
	void showFiles()  {
		// get selected items
		Object [] selected = list2.getSelectedValues();
		if (selected == null) return;
		
		// show each files
		for (int i=0; i<selected.length; ++i)
			{
			final JDupFinderFile file = (JDupFinderFile) selected[i];
			file.frame = new JDupFinderShowFrame(file.filename);
			file.frame.setVisible(true);
			}
	}

	
	void updateList2(ListSelectionEvent e)  {
		// check if there are more events to come
		if (e != null && e.getValueIsAdjusting()) return;
		
		// index of selected item
		int index = list1.getSelectedIndex();
		if (index < 0) 
			{
			list2.setListData(new Vector());
			return;
			}
		
		// get and set the data for the second list
		TreeSet set = (TreeSet) duplicates.elementAt(index);
		list2.setListData(set.toArray());
		
		// select all but first
		int [] select = new int[set.size()-1];
		for (int i=0; i<select.length; ++i) select[i] = i;
		list2.setSelectedIndices(select);
	}

}
